
const textElement = document.getElementById("text")
const configureElement = document.getElementById("configure")
const readersHelp = document.getElementById("readersHelp")
const styleElement = document.getElementById("confettiStyle")
const customElement = document.getElementById("customEmoji")
const realtimeChannel = window.webxdc.joinRealtimeChannel()

var confettiStyle = 'confetti'
var customString = '🏆 YES!'
var initDone = false
var isEditable = true
var isTouch = false


function localConfetti() {
    throwConfetti(confettiStyle, customString)
}


function docTapped(e) {
    if (e.type === 'mousedown' && isTouch) { return; }
    if (e.type === 'touchstart') { isTouch = true; }
    setTimeout(() => (isTouch = false), 1000) // avoid issue when user switch between touch and mouse on hybrids

    // customElement is selected - do not throw old confetti setting
    if (document.activeElement === customElement) { return; }

    localConfetti()
    realtimeChannel.send(new Uint8Array(1))
}


// inspired by https://stephenhaney.com/2020/get-contenteditable-plaintext-with-correct-linebreaks/
var parseNewValue = ''
var parseIsOnFreshLine = true
function parseChildNodesForValueAndLines(childNodes) {
    for (let i = 0; i < childNodes.length; i++) {
        const childNode = childNodes[i];
        if (childNode.nodeName === 'BR') {
            parseNewValue += '\n'
            parseIsOnFreshLine = true
            continue;
        }

        if (childNode.nodeName === 'DIV' && parseIsOnFreshLine === false) {
            parseNewValue += '\n'
        }

        parseIsOnFreshLine = false

        if (childNode.nodeType === 3 && childNode.textContent) {
            parseNewValue += childNode.textContent
        }

        parseChildNodesForValueAndLines(childNode.childNodes)
    }
}


function editDone(e) {
    parseNewValue = ''
    parseIsOnFreshLine = true
    parseChildNodesForValueAndLines(textElement.cloneNode(true).childNodes)
    var text = parseNewValue.trim()

    if (text == '') {
        // set empty text to something, otherwise, one gets a hard time to edit ever again
        // (on playing around, one easily taps outside accidentally)
        text = "Congrats!"
        textElement.textContent = text // this disturbs cursor, do not do this usually
    }

    window.webxdc.sendUpdate({
        payload: {
            text: text,
            sender: window.webxdc.selfAddr
        },
        summary: text.split("\n")[0]
    }, "")
}


function styleSelected(e) {
    if (confettiStyle != styleElement.value || customString != customElement.value) {
        confettiStyle = styleElement.value
        customString = customElement.value

        customElement.style.display = confettiStyle == 'custom' ? "inline" : "none"

        window.webxdc.sendUpdate({
            payload: {
                confettiStyle: confettiStyle,
                customString: customString,
                sender: window.webxdc.selfAddr
            }
        }, "")
    }
}


var isReadOnly = false
function setReadOnly() {
    if (!isReadOnly) {
        isReadOnly = true
        textElement.classList.add('noSelect')
        readersHelp.style.display = "block"
        setTimeout(() => {
            readersHelp.classList.add('fadeIn')
        }, 4000);
    }
}

window.webxdc.setUpdateListener(function (update) {

    if (update.payload.sender != window.webxdc.selfAddr) {
        // another peer is editing, this one can no longer edit
        textElement.contentEditable = false
        textElement.style.cursor = "default"
        configureElement.style.display = "none"
        setReadOnly()
        isEditable = false
    }

    if (update.payload.sender != window.webxdc.selfAddr || !initDone) {
        // take over configuration from other peer or during initialisation
        if (typeof update.payload.text !== 'undefined') {
            textElement.textContent = update.payload.text
        }

        if (typeof update.payload.confettiStyle !== 'undefined') {
            confettiStyle = update.payload.confettiStyle
            customString = update.payload.customString
        }
    }

    if (initDone) {
        localConfetti() // this is also visual feedback from "blur" below
    }

})
.then(() => {

    initDone = true
    textElement.style.display = "block"
    if (isEditable) {
        textElement.contentEditable = true
        textElement.style.cursor = "text"
        configureElement.style.display = "block"
    } else {
        setReadOnly()
    }

    textElement.addEventListener("blur", editDone)
    styleElement.addEventListener("change", styleSelected);
    styleElement.value = confettiStyle
    customElement.style.display = confettiStyle == 'custom' ? "inline" : "none"
    customElement.value = customString
    customElement.addEventListener("blur", styleSelected)

    realtimeChannel.setListener((data) => {
        localConfetti()
    })

    // "click" would work on both, but have a 300ms delay
    document.addEventListener('mousedown', docTapped)
    document.addEventListener('touchstart', docTapped)

    // initial confetti on opening is synced as well
    const initialDelay = 333
    setTimeout(() => {
        localConfetti()
        realtimeChannel.send(new Uint8Array(1))
    }, initialDelay)


    setTimeout(() => {
        localConfetti()
        realtimeChannel.send(new Uint8Array(1))
    }, initialDelay + 500 + Math.floor(Math.random() * 2000));

    setTimeout(() => {
        localConfetti()
        realtimeChannel.send(new Uint8Array(1))
    }, initialDelay + 500 + Math.floor(Math.random() * 2000));

})
